@@ -1,27 +1,28 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright (c) 2012 Eric S. Raymond <esr@thyrsus.com>
 # SPDX-License-Identifier: BSD-2-Clause
-#
-# This script contains git porcelain and porcelain byproducts.
-# Requires either Python 2.6, or 2.5 with the simplejson library installed
-# or Python 3.x.
-#
-# usage: irkerhook.py [-V] [-n] [--variable=value...] [commit_id...]
-#
-# This script is meant to be run in an update or post-commit hook.
-# Try it with -n to see the notification dumped to stdout and verify
-# that it looks sane. With -V this script dumps its version and exits.
-#
-# See the irkerhook manual page in the distribution for a detailed
-# explanation of how to configure this hook.
-from __future__ import print_function
+'''
+This script contains git porcelain and porcelain byproducts.
+Requires either Python 2.6, or 2.5 with the simplejson library installed
+or Python 3.x.
 
-# The default location of the irker proxy, if the project configuration
-# does not override it.
-#
+usage: irkerhook.py [-V] [-n] [--variable=value...] [commit_id...]
+
+This script is meant to be run in an update or post-commit hook.
+Try it with -n to see the notification dumped to stdout and verify
+that it looks sane. With -V this script dumps its version and exits.
+
+See the irkerhook manual page in the distribution for a detailed
+explanation of how to configure this hook.
+
+The default location of the irker proxy, if the project configuration
+does not override it.
+'''
 # SPDX-License-Identifier: BSD-2-Clause
 from __future__ import print_function, absolute_import
 
+# pylint: disable=line-too-long,invalid-name,missing-function-docstring,missing-class-docstring,no-else-break,no-else-return,too-many-instance-attributes,too-many-locals,too-many-branches,too-many-statements,redefined-outer-name,import-outside-toplevel,raise-missing-from
+
 default_server = "localhost"
 IRKER_PORT = 6659
 
@@ -43,10 +44,15 @@ default_channels = u"irc://chat.freenode.net/#commits"
 # No user-serviceable parts below this line:
 #
 
-version = "2.19"
+version = "2.21"
 
+# pylint: disable=multiple-imports,wrong-import-position
 import os, sys, socket, subprocess, locale, datetime, re
-from pipes import quote as shellquote
+
+try
+    from shlex import quote as shellquote
+except ImportError:
+    from pipes import quote as shellquote
 
 try:
     from urllib2 import urlopen, HTTPError
@@ -60,6 +66,7 @@ except ImportError:
     import json
 
 if sys.version_info.major == 2:
+    # pylint: disable=undefined-variable
     string_type = unicode
 else:
     string_type = str
@@ -67,6 +74,7 @@ else:
 try:
     getstatusoutput = subprocess.getstatusoutput
 except AttributeError:
+    # pylint: disable=import-error
     import commands
     getstatusoutput = commands.getstatusoutput
 
@@ -76,6 +84,7 @@ def do(command):
     else:
         return getstatusoutput(command)[1]
 
+# pylint: disable=too-few-public-methods
 class Commit:
     def __init__(self, extractor, commit):
         "Per-commit data."
@@ -87,8 +96,10 @@ class Commit:
         self.files = None
         self.logmsg = None
         self.url = None
+        self.author_name = None
         self.author_date = None
         self.commit_date = None
+        self.id = None
         self.__dict__.update(extractor.__dict__)
 
         if sys.version_info.major == 2:
@@ -99,9 +110,11 @@ class Commit:
 
     def __str__(self):
         "Produce a notification string from this commit."
-        if self.urlprefix.lower() == "none":
+        # pylint: disable=no-member
+        if not self.urlprefix:
             self.url = ""
         else:
+            # pylint: disable=no-member
             urlprefix = urlprefixmap.get(self.urlprefix, self.urlprefix)
             webview = (urlprefix % self.__dict__) + self.commit
             try:
@@ -128,6 +141,7 @@ class Commit:
                     self.url = ""
             except IOError:
                 self.url = ""
+        # pylint: disable=no-member
         res = self.template % self.__dict__
         return string_type(res, 'UTF-8') if not isinstance(res, string_type) else res
 
@@ -154,7 +168,7 @@ class GenericExtractor:
         self.filtercmd = None
         # Color highlighting is disabled by default.
         self.color = None
-        self.bold = self.green = self.blue = self.yellow = ""
+        self.bold = self.green = self.blue = self.yellow = self.red = ""
         self.brown = self.magenta = self.cyan = self.reset = ""
     def activate_color(self, style):
         "IRC color codes."
@@ -167,6 +181,7 @@ class GenericExtractor:
             self.bold = '\x02'
             self.green = '\x0303'
             self.blue = '\x0302'
+            self.red = '\x0304'
             self.red = '\x0305'
             self.yellow = '\x0307'
             self.brown = '\x0305'
@@ -192,13 +207,13 @@ class GenericExtractor:
             ln += 1
             if line.startswith("#") or not line.strip():
                 continue
-            elif line.count('=') != 1:
-                sys.stderr.write('"%s", line %d: missing = in config line\n' \
+            if line.count('=') != 1:
+                sys.stderr.write('%s:%d: missing = in config line\n' \
                                  % (conf, ln))
                 continue
             fields = line.split('=')
             if len(fields) != 2:
-                sys.stderr.write('"%s", line %d: too many fields in config line\n' \
+                sys.stderr.write('%s:%d: too many fields in config line\n' \
                                  % (conf, ln))
                 continue
             variable = fields[0].strip()
@@ -220,9 +235,9 @@ class GenericExtractor:
                     setattr(self, key, val)
         for (key, val) in self.__dict__.items():
             if key in GenericExtractor.booleans:
-                if type(val) == type("") and val.lower() == "true":
+                if isinstance(val, str) and val.lower() == "true":
                     setattr(self, key, True)
-                elif type(val) == type("") and val.lower() == "false":
+                elif isinstance(val, str) and val.lower() == "false":
                     setattr(self, key, False)
             elif key in GenericExtractor.numerics:
                 setattr(self, key, int(val))
@@ -293,6 +308,7 @@ class GitExtractor(GenericExtractor):
                 here = os.path.dirname(here)
         # Get overrides
         self.do_overrides()
+    # pylint: disable=no-self-use
     def head(self):
         "Return a symbolic reference to the tip commit of the current branch."
         return "HEAD"
@@ -345,8 +361,10 @@ class SvnExtractor(GenericExtractor):
         self.project = os.path.basename(self.repository)
         self.template = '%(bold)s%(project)s%(reset)s: %(green)s%(author)s%(reset)s %(repo)s * %(bold)s%(rev)s%(reset)s / %(bold)s%(files)s%(reset)s: %(logmsg)s %(brown)s%(url)s%(reset)s'
         self.urlprefix = "viewcvs"
+        self.id = None
         self.load_preferences(os.path.join(self.repository, "irker.conf"))
         self.do_overrides()
+    # pylint: disable=no-self-use
     def head(self):
         sys.stderr.write("irker: under svn, hook requires a commit argument.\n")
         raise SystemExit(1)
@@ -369,6 +387,7 @@ class HgExtractor(GenericExtractor):
     def is_repository(directory):
         return has(directory, [".hg"])
     def __init__(self, arguments):
+        from mercurial.encoding import unifromlocal, unitolocal
         # This fiddling with arguments is necessary since the Mercurial hook can
         # be run in two different ways: either directly via Python (in which
         # case hg should be pointed to the hg_hook function below) or as a
@@ -377,7 +396,7 @@ class HgExtractor(GenericExtractor):
         # ui objects from Mercurial, in the second case, we have to create them
         # from the root path.
         self.repository = None
-        if arguments and type(arguments[0]) == type(()):
+        if arguments and isinstance(arguments[0], tuple):
             # Called from hg_hook function
             ui, self.repository = arguments[0]
             arguments = []  # Should not be processed further by do_overrides
@@ -385,35 +404,42 @@ class HgExtractor(GenericExtractor):
             # Called from command line: create repo/ui objects
             from mercurial import hg, ui as uimod
 
-            repopath = '.'
+            repopath = b'.'
             for tok in arguments:
                 if tok.startswith('--repository='):
-                    repopath = tok[13:]
+                    repopath = unitolocal(tok[13:])
             ui = uimod.ui()
-            ui.readconfig(os.path.join(repopath, '.hg', 'hgrc'), repopath)
+            ui.readconfig(os.path.join(repopath, b'.hg', b'hgrc'), repopath)
             self.repository = hg.repository(ui, repopath)
 
         GenericExtractor.__init__(self, arguments)
         # Extract global values from the hg configuration file(s)
-        self.project = ui.config('irker', 'project')
-        self.repo = ui.config('irker', 'repo')
-        self.server = ui.config('irker', 'server')
-        self.channels = ui.config('irker', 'channels')
-        self.email = ui.config('irker', 'email')
-        self.tcp = str(ui.configbool('irker', 'tcp'))  # converted to bool again in do_overrides
-        self.template = ui.config('irker', 'template') or '%(bold)s%(project)s:%(reset)s %(green)s%(author)s%(reset)s %(repo)s:%(yellow)s%(branch)s%(reset)s * %(bold)s%(rev)s%(reset)s / %(bold)s%(files)s%(reset)s: %(logmsg)s %(brown)s%(url)s%(reset)s'
-        self.tinyifier = ui.config('irker', 'tinyifier') or default_tinyifier
-        self.color = ui.config('irker', 'color')
-        self.urlprefix = (ui.config('irker', 'urlprefix') or
-                          ui.config('web', 'baseurl') or '')
+        self.project = unifromlocal(ui.config(b'irker', b'project') or b'')
+        self.repo = unifromlocal(ui.config(b'irker', b'repo') or b'')
+        self.server = unifromlocal(ui.config(b'irker', b'server') or b'')
+        self.channels = unifromlocal(ui.config(b'irker', b'channels') or b'')
+        self.email = unifromlocal(ui.config(b'irker', b'email') or b'')
+        self.tcp = str(ui.configbool(b'irker', b'tcp'))  # converted to bool again in do_overrides
+        self.template = unifromlocal(ui.config(b'irker', b'template') or b'')
+        if not self.template:
+            self.template = '%(bold)s%(project)s:%(reset)s %(green)s%(author)s%(reset)s %(repo)s:%(yellow)s%(branch)s%(reset)s * %(bold)s%(rev)s%(reset)s / %(bold)s%(files)s%(reset)s: %(logmsg)s %(brown)s%(url)s%(reset)s'
+        self.tinyifier = unifromlocal(ui.config(
+                b'irker', b'tinyifier',
+                default=default_tinyifier.encode('utf-8')))
+        self.color = unifromlocal(ui.config(b'irker', b'color') or b'')
+        self.urlprefix = unifromlocal(ui.config(
+                b'irker', b'urlprefix', default=ui.config(b'web', b'baseurl')))
         if self.urlprefix:
             # self.commit is appended to this by do_overrides
-            self.urlprefix = self.urlprefix.rstrip('/') + '/rev/'
-        self.cialike = ui.config('irker', 'cialike')
-        self.filtercmd = ui.config('irker', 'filtercmd')
+            self.urlprefix = (
+                self.urlprefix.rstrip('/')
+                + '/%s/rev/' % unifromlocal(self.repository.root).rstrip('/'))
+        self.cialike = unifromlocal(ui.config(b'irker', b'cialike') or b'')
+        self.filtercmd = unifromlocal(ui.config(b'irker', b'filtercmd') or b'')
         if not self.project:
-            self.project = os.path.basename(self.repository.root.rstrip('/'))
+            self.project = os.path.basename(unifromlocal(self.repository.root).rstrip('/'))
         self.do_overrides()
+    # pylint: disable=no-self-use
     def head(self):
         "Return a symbolic reference to the tip commit of the current branch."
         return "-1"
@@ -421,19 +447,21 @@ class HgExtractor(GenericExtractor):
         "Make a Commit object holding data for a specified commit ID."
         from mercurial.node import short
         from mercurial.templatefilters import person
-        node = self.repository.lookup(commit_id)
-        commit = Commit(self, short(node))
+        from mercurial.encoding import unifromlocal, unitolocal
+        if isinstance(commit_id, str) and not isinstance(commit_id, bytes):
+            commit_id = unitolocal(commit_id)
+        ctx = self.repository[commit_id]
+        commit = Commit(self, unifromlocal(short(ctx.hex())))
         # Extract commit-specific values from a "context" object
-        ctx = self.repository.changectx(node)
         commit.rev = '%d:%s' % (ctx.rev(), commit.commit)
-        commit.branch = ctx.branch()
-        commit.author = person(ctx.user())
+        commit.branch = unifromlocal(ctx.branch())
+        commit.author = unifromlocal(person(ctx.user()))
         commit.author_date = \
             datetime.datetime.fromtimestamp(ctx.date()[0]).strftime('%Y-%m-%d %H:%M:%S')
-        commit.logmsg = ctx.description()
+        commit.logmsg = unifromlocal(ctx.description())
         # Extract changed files from status against first parent
         st = self.repository.status(ctx.p1().node(), ctx.node())
-        commit.files = ' '.join(st.modified + st.added + st.removed)
+        commit.files = unifromlocal(b' '.join(st.modified + st.added + st.removed))
         return commit
 
 def hg_hook(ui, repo, **kwds):
@@ -484,7 +512,7 @@ def ship(extractor, commit, debug):
            and extractor.cialike.lower() != "none" \
            and len(metadata.files) > int(extractor.cialike):
         files = metadata.files.split()
-        dirs = set([d.rpartition('/')[0] for d in files])
+        dirs = {d.rpartition('/')[0] for d in files}
         if len(dirs) == 1:
             metadata.files = "(%s files)" % (len(files),)
         else:
